@@ -31,6 +31,7 @@ pub struct App {
     playable_cards: Vec<CardIdx>,
     scores: HashMap<CardIdx, usize>,
     dues: HashMap<CardIdx, DateTime<Utc>>,
+    circulating: Vec<CardIdx>,
     /// This determines whether the card should show the
     /// question or the answer.
     pub flipped: bool,
@@ -48,6 +49,7 @@ impl App {
             playable_cards: (0..cards.len()).collect(),
             cards,
             scores: HashMap::new(),
+            circulating: Vec::new(),
             flipped: false,
             dues: HashMap::new(),
             opts,
@@ -64,6 +66,7 @@ impl App {
     pub fn order_playables(&mut self) {
         self.playable_cards.sort_unstable_by_key(|card| {
             (
+                self.circulating.contains(card),
                 *self.scores.get(card).unwrap_or(&usize::MAX),
                 *self.dues.get(card).unwrap_or(&chrono::MAX_DATETIME),
             )
@@ -90,6 +93,12 @@ impl App {
         &self.scores
     }
 
+    /// Returns a copy of cards which are mandatorily in
+    /// circulation until answered correctly
+    pub fn circulating(&self) -> &[CardIdx] {
+        &self.circulating
+    }
+
     /// Flips the current card to show the other side.
     pub fn flip(&mut self) {
         self.flipped ^= true;
@@ -119,6 +128,7 @@ impl App {
         // Take the memory because otherwise we will have borrowed
         // self twice.
         let mut playable_cards = mem::take(&mut self.playable_cards);
+        let mut circulating = mem::take(&mut self.circulating);
 
         // Remove all the cards which we don't need
         playable_cards.retain(|x| !self.current_card_is_finished(*x));
@@ -179,11 +189,20 @@ impl App {
         let entry = self.scores.entry(self.card).or_default();
         if val > 0 {
             *entry = entry.saturating_add(val.try_into().unwrap_or(0));
+            if let Some(x) =
+                self.circulating.iter().position(|x| *x == self.card)
+            {
+                self.circulating.swap_remove(x);
+            }
         } else {
             if self.opts.zeroize {
                 *entry = 0;
             } else {
-                *entry = entry.saturating_sub(val.try_into().unwrap_or(0));
+                *entry =
+                    entry.saturating_sub((0 - val).try_into().unwrap_or(0));
+                if !self.circulating.contains(&self.card) {
+                    self.circulating.push(self.card)
+                }
             }
         }
         self.reset_current_card_duration();
@@ -194,10 +213,14 @@ impl App {
     pub fn current_card_is_finished(&self, card: usize) -> bool {
         let now = Utc::now();
 
-        if let Some(val) = self.dues.get(&card) {
-            now < *val
-        } else {
+        if self.circulating.contains(&card) {
             false
+        } else {
+            if let Some(val) = self.dues.get(&card) {
+                now < *val
+            } else {
+                false
+            }
         }
     }
 
@@ -248,6 +271,13 @@ impl App {
                 }
             }
 
+            for card in scores.circulating {
+                let pos = self.cards().iter().position(|x| x == card.as_ref());
+                if let Some(pos) = pos {
+                    self.circulating.push(pos);
+                }
+            }
+
             self.retain_undue();
         }
     }
